import moment from 'moment'

const _ALL_RECEIPTS = 'receipts'
const _PREFIX = 'receipt-'

export default class Receipt {
  /*
   receipt = {
    id: 'foobar'
    items = [
      {
        name: 'name1',
        price: 0.1,
        category: 'cat1',
        order: 1
      },
      {
        name: 'name2',
        price: 0.2,
        category: 'cat2',
        order: 2
      },
      ...
      {
        name: 'nameN',
        price: 0.N,
        category: 'cat3',
        order: N
      }
    ]
   }
   */
  constructor (id, items, when, where) {
    this.id = id
    this.items = items.map(({name, price, category, order}) => {
      return {
        name,
        category,
        order,
        price: typeof price === 'string' ? Number.parseFloat(price) : price
      }
    })
    this.when = when ? moment(when) : moment()
    this.where = where || 'unknown'
    this.timestamp = this.when.unix()
  }

  get total () {
    return this.items.map(item => item.price).reduce((acc, val) => acc + val, 0)
  }

  get catTotals () {
    let catTotals = {}
    for (let item of this.items) {
      const safeCatTitle = item.category || 'uncategorized'
      catTotals[safeCatTitle] = (catTotals[safeCatTitle] || 0) + item.price
    }
    return catTotals
  }

  get categories () {
    return this.items.map(item => item.category)
  }

  commitTo (ls) {
    if (!Receipt.enumerateAll(ls).includes(this.id)) {
      const tmpIndex = JSON.parse(ls.get(_ALL_RECEIPTS, '[]'))
      tmpIndex.push(this.id)
      ls.set(_ALL_RECEIPTS, JSON.stringify(tmpIndex))
    }
    ls.set(`${_PREFIX}${this.id}`, JSON.stringify({
      id: this.id,
      items: this.items,
      where: this.where,
      when: this.when.format()
    }))
  }

  removeFrom (ls) {
    let tmpIndex = JSON.parse(ls.get(_ALL_RECEIPTS, '[]'))
    tmpIndex = tmpIndex.filter(i => i !== this.id)
    ls.set(_ALL_RECEIPTS, JSON.stringify(tmpIndex))
    ls.remove(`${_PREFIX}${this.id}`)
  }

  addDefaultCategories () {
    this.items.map(item => Object.assign(item, {category: ''}))
  }

  static getFrom (ls, id) {
    const stored = JSON.parse(ls.get(`${_PREFIX}${id}`))
    if (!stored) {
      return null
    }
    return new Receipt(stored.id, stored.items, stored.when, stored.where)
  }

  static enumerateAll (ls) {
    const stored = JSON.parse(ls.get(_ALL_RECEIPTS))
    if (!Array.isArray(stored)) {
      ls.set(_ALL_RECEIPTS, '[]')
      return []
    }
    return stored
  }
}
